@@ -47,24 +47,147 @@
 !! *************************************************************************************************
 
 MODULE SHARED_DATA
+    USE ISO_Fortran_env
+    USE ISO_C_BINDING
     IMPLICIT NONE
-    REAL(8),PARAMETER :: M_PI         = 3.141592653589793238462643383280
-    REAL(8),PARAMETER :: M_1_SQRT_2PI = 0.398942280401432677939946059934 ! 1/(sqrt(2*pi))
+    !> Double precision real numbers, 15 digits, range 10⁻³⁰⁷ to 10³⁰⁷-1; 64 bits
+    integer, parameter :: dp = selected_real_kind(15, 307)
+    
+    REAL(dp),PARAMETER :: M_PI         = 3.141592653589793238462643383280_dp
+    REAL(dp),PARAMETER :: M_1_SQRT_2PI = 0.398942280401432677939946059934_dp ! 1/(sqrt(2*pi))
+
+interface
+    subroutine GetRNGstate() bind(C,name='GetRNGstate')
+    end subroutine GetRNGstate
+
+    subroutine PutRNGstate() bind(C,name='PutRNGstate')
+    end subroutine PutRNGstate
+
+    function norm_rand() bind(C,name='norm_rand')
+     import
+     implicit none
+     real(C_DOUBLE) norm_rand
+    end function norm_rand
+
+    function unif_rand() bind(C,name='unif_rand')
+     import
+     implicit none
+     real(C_DOUBLE) unif_rand
+    end function unif_rand
+
+end interface
+    public GetRNGstate, PutRNGstate
+    public norm_rand, unif_rand
+    public PERM
+
+CONTAINS
+
+! *****************************************************************************
+! *****************************************************************************
+SUBROUTINE KGAUSSv(x,d,n,K)
+    IMPLICIT NONE
+    !! ********************************
+    !! Multiplicative Gaussian Kernel (vectorized)
+    !! ********************************
+    INTEGER,INTENT(IN)  :: n,d
+    REAL(dp),INTENT(IN)  :: x(d,n)
+    REAL(dp),INTENT(OUT) :: K(n)
+    K = (M_1_SQRT_2PI**d)*EXP(-0.5*(SUM(x**2,DIM=1)))
+END SUBROUTINE KGAUSSv
+
+! *****************************************************************************
+! *****************************************************************************
+
+FUNCTION BOOT(N,size)
+! gives a random sample with replacement of length size from the first N integers
+    INTEGER,INTENT(IN) :: N,size
+    INTEGER :: BOOT(size)
+    REAL(dp) :: u(size)
+    CALL randunif(u,size)
+    BOOT = int(u*N)+1   ! random number between fra 1 e N
+END FUNCTION BOOT
+
+!! ****************************************************************************
+SUBROUTINE permute(x,n)
+    ! random permutation of a vector x
+    use, intrinsic :: iso_c_binding
+    IMPLICIT NONE
+    INTEGER(C_INT),INTENT(IN):: n
+    REAL(C_DOUBLE),INTENT(INOUT):: x(n)
+    INTEGER:: IND(n)
+    IND = PERM(n)
+    x   = x(IND)
+END SUBROUTINE permute
+!! *****************************************************************************
+
+subroutine randnorm(x,n)
+! generates n standard normal random numbers
+   use ISO_C_BINDING
+   implicit none
+   INTEGER,INTENT(IN):: n
+   real(C_DOUBLE),intent(out):: x(n)
+   integer :: i
+   call GetRNGstate();
+    do i=1,n
+        x(i) = norm_rand()
+    end do
+   call PutRNGstate();
+end subroutine randnorm
+
+! *****************************************************************************
+
+subroutine randunif(x,n)
+! generates n continuous uniform random numbers in [0,1]
+   use ISO_C_BINDING
+   implicit none
+   INTEGER,INTENT(IN):: n
+   real(C_DOUBLE),intent(out):: x(n)
+   integer :: i
+   call GetRNGstate();
+    do i=1,n
+        x(i) = unif_rand()
+    end do
+   call PutRNGstate();
+end subroutine randunif
+
+!! ****************************************************************************
+
+    FUNCTION BOOTR(N,size)
+! sampling without replacement
+! gives a random sample without replacement of length size from the first N integers
+! generalizes the function PERM
+
+        INTEGER,INTENT(IN) :: N,size
+        INTEGER :: BOOTR(size)
+        INTEGER :: B(N)
+        REAL(dp) :: u(N)
+        INTEGER :: i,j,k
+
+        B     =  (/(i,i=1,N)/)
+        BOOTR = 0  ! vettore degli n numeri estratti
+        CALL randunif(u,N)
+        do i = N,(N-size+1),-1
+            j = int(u(i)*i)+1   ! numero casuale compreso fra 1 e i
+            BOOTR((N-i+1)) = B(j)
+            k = B(j)
+            B(j)=B(i)
+            B(i)=k
+        enddo
+    END FUNCTION BOOTR
 
-    CONTAINS
 !! ****************************************************************************
     FUNCTION PERM(N)
         ! GIVES A RANDOM PERMUTATION OF THE FIRST N INTEGERS
 
         INTEGER,INTENT(IN) :: N
-        INTEGER,DIMENSION(N) :: PERM
-        INTEGER,DIMENSION(N) :: B
-        REAL(8) :: u(N)
+        INTEGER :: PERM(N)
+        INTEGER :: B(N)
+        REAL(dp) :: u(N)
         INTEGER :: i,j,k
 
         B   =  (/(i,i=1,N)/)
         PERM = 0  ! vettore degli n numeri estratti
-        CALL RANDOM_NUMBER(u)
+        CALL randunif(u,N)
         do i = N,1,-1
             j = int(u(i)*i)+1   ! numero casuale compreso fra 1 e i
             PERM(i) = B(j)
@@ -78,7 +201,7 @@ MODULE SHARED_DATA
 SUBROUTINE DNORMF(X,N,OUT)
     IMPLICIT NONE
     INTEGER:: N
-    REAL(8) :: X(N),OUT(N)
+    REAL(dp) :: X(N),OUT(N)
     OUT = EXP(-.5*X**2)*M_1_SQRT_2PI
 END SUBROUTINE DNORMF
 !! *************************************************************************************************
@@ -86,11 +209,10 @@ END SUBROUTINE DNORMF
 SUBROUTINE SRHOBIVA(TX,TY,NUNI,NX,NY,T,NBIV,S,nor)
     IMPLICIT NONE
     INTEGER,INTENT(IN):: NX,NY,NUNI,NBIV,TX(NX,2),TY(NY,2),T(NX,NY),nor
-    REAL(8),INTENT(OUT):: S
-
-    REAL(8)  :: PX(NX),PY(NY),P(NX,NY),smax
+    REAL(dp),INTENT(OUT):: S
+    REAL(dp) :: PX(NX),PY(NY),P(NX,NY),smax
     INTEGER :: ix,iy
-    S = 0.
+    S = 0.0_dp
     PX = DBLE(TX(:,2))/NUNI
     PY = DBLE(TY(:,2))/NUNI
     P  = DBLE(T)/NBIV
@@ -104,13 +226,10 @@ SUBROUTINE SRHOBIVA(TX,TY,NUNI,NX,NY,T,NBIV,S,nor)
         smax = MAX(1 - SUM(PX**1.5), 1 - SUM(PY**1.5))
         S=S/smax
     ENDIF
-!    call dblepr("S", -1, S, 1)
-!    call intpr("nor_SB", -1, nor, 1)
 END SUBROUTINE SRHOBIVA
 !! *************************************************************************************************
 
 SUBROUTINE TABFD(X,Y,N,TX,TY,T)
-!!DEC$ ATTRIBUTES DLLEXPORT,C,REFERENCE,ALIAS:'tabfd' :: tabfd
 !! ******************************************************************************
 !! TWO WAY frequency table for integer data
 !! ******************************************************************************
@@ -209,20 +328,25 @@ SUBROUTINE TABF(X,N,T)
         T(:,:)=DUM(1:cnt,:)
     ENDIF
 END SUBROUTINE TABF
-!! *************************************************************************************************
+
+
+
+!! ****************************************************************************
+!! ****************************************************************************
 END MODULE SHARED_DATA
-!! *************************************************************************************************
+!! ****************************************************************************
+!! ****************************************************************************
 
 SUBROUTINE MBBOOT(X,N,B,l,M)
-
+    USE SHARED_DATA
     IMPLICIT NONE
     INTEGER,INTENT(IN):: N,B,l
-    REAL(8),INTENT(IN):: X(N)
-    REAL(8),INTENT(OUT):: M(N,B)
+    REAL(dp),INTENT(IN):: X(N)
+    REAL(dp),INTENT(OUT):: M(N,B)
 
     INTEGER:: nblocks,ind(N),indmat(N-l+1,l),i
     INTEGER,allocatable:: indblock(:),indx(:,:)
-    REAL(8),allocatable:: M2(:,:)
+    REAL(dp),allocatable:: M2(:,:)
     nblocks = int(N/l) + 1
     ALLOCATE(indblock(nblocks*B),indx(nblocks*l*B,1),M2(nblocks*l, B))
     ind      = (/(i,i=1,N)/)
@@ -232,30 +356,6 @@ SUBROUTINE MBBOOT(X,N,B,l,M)
     M2       = RESHAPE(X(indx(:,1)),(/nblocks*l, B/))
     M        = M2(1:N,:)
 CONTAINS
-    FUNCTION BOOT(N,size)
-        ! GIVES A RESAMPLE WITH REPLACEMENT OF LENGTH size OF THE FIRST N INTEGERS
-        INTEGER,INTENT(IN) :: N,size
-        INTEGER,DIMENSION(size) :: BOOT
-        REAL(8) :: u(size)
-        CALL init_random_seed()
-        CALL RANDOM_NUMBER(u)
-        BOOT = int(u*N)+1   ! numero casuale compreso fra 1 e N
-    END FUNCTION BOOT
-
-    SUBROUTINE init_random_seed()
-        INTEGER :: i, n, clock
-        INTEGER, DIMENSION(:), ALLOCATABLE :: seed
-
-        CALL RANDOM_SEED(size = n)
-        ALLOCATE(seed(n))
-
-        CALL SYSTEM_CLOCK(COUNT=clock)
-
-        seed = clock + 37 * (/ (i - 1, i = 1, n) /)
-        CALL RANDOM_SEED(PUT = seed)
-
-        DEALLOCATE(seed)
-    END SUBROUTINE
 
     FUNCTION EMBED(X,j,d)
 
@@ -265,8 +365,8 @@ CONTAINS
     !        d : embedding dimension
         implicit none
         integer,intent(IN):: j,d
-        integer,dimension(:),intent(IN) :: X
-        integer,dimension(size(X,1)-(d-1)*j,d):: EMBED
+        integer,intent(IN) :: X(:)
+        integer:: EMBED(size(X,1)-(d-1)*j,d)
      	
         integer :: npun,h
         npun = size(X,1)-(d-1)*j
@@ -280,7 +380,7 @@ END SUBROUTINE MBBOOT
 
 !! *************************************************************************************************
 SUBROUTINE SSBIV(X,Y,N,nlag,S,nor)
-!!DEC$ ATTRIBUTES DLLEXPORT,C,REFERENCE,ALIAS:'ssbiv' :: ssbiv
+
 
 !! **************************************************************************
 !! Implementation of the entropy-based dependence measure
@@ -304,13 +404,13 @@ SUBROUTINE SSBIV(X,Y,N,nlag,S,nor)
     USE SHARED_DATA
     IMPLICIT NONE
     INTEGER,intent(IN):: N,nlag,X(N),Y(N),nor
-    REAL(8),intent(OUT):: S(2*nlag+1)
+    REAL(dp),intent(OUT):: S(2*nlag+1)
 
     INTEGER,allocatable:: TX(:,:),TY(:,:),T(:,:)
-    REAL(8)  :: dum
+    REAL(dp)  :: dum
     INTEGER :: k,nx,ny
 
-    S = 999
+    S = 999_dp
     CALL TABFD(X,Y,N,TX,TY,T)
         nx = SIZE(TX,1)
         ny = SIZE(TY,1)
@@ -334,7 +434,7 @@ END SUBROUTINE SSBIV
 
 !! *************************************************************************************************
 SUBROUTINE SSBIVB(X,Y,N,nlag,B,S,M,STAT,nor)
-!!DEC$ ATTRIBUTES DLLEXPORT,C,REFERENCE,ALIAS:'ssbivb' :: ssbivb
+
 
 !! ******************************************************************************
 !! PERMUTATION VERSION OF SSBIV
@@ -357,25 +457,29 @@ SUBROUTINE SSBIVB(X,Y,N,nlag,B,S,M,STAT,nor)
 
 INTERFACE
     SUBROUTINE SSBIV(X,Y,N,nlag,S,nor)
+        USE SHARED_DATA
+        USE ISO_Fortran_env
         INTEGER,INTENT(IN):: N,nlag,X(N),Y(N),nor
-        REAL(8),intent(OUT):: S(2*nlag+1)
+        REAL(dp),intent(OUT):: S(2*nlag+1)
     END SUBROUTINE SSBIV
     SUBROUTINE SSBIV2(X,Y,N,nlag,S,nor)
+        USE SHARED_DATA
+        USE ISO_Fortran_env
         INTEGER,INTENT(IN):: N,nlag,X(N),Y(N),nor
-        REAL(8),intent(OUT):: S(2*nlag+1)
+        REAL(dp),intent(OUT):: S(2*nlag+1)
     END SUBROUTINE SSBIV2
 END INTERFACE
 
     INTEGER,INTENT(IN):: N,nlag,X(N),Y(N),B,STAT,nor
-    REAL(8),intent(OUT):: S(2*nlag+1),M(2*nlag+1,B)
+    REAL(dp),intent(OUT):: S(2*nlag+1),M(2*nlag+1,B)
 
-    REAL(8) :: dum(2*nlag+1)
+    REAL(dp) :: dum(2*nlag+1)
     INTEGER:: ind(N),XB(N),YB(N)
     INTEGER:: i
 
-    S    = 0.;
-    M    = 0.;
-    dum  = 0.;
+    S    = 0.0_dp;
+    M    = 0.0_dp;
+    dum  = 0.0_dp;
     ind  = 0 ;
     XB   = 0 ;
     YB   = 0 ;
@@ -406,7 +510,7 @@ END SUBROUTINE SSBIVB
 !! *************************************************************************************************
 
 SUBROUTINE SSBIV2(X,Y,N,nlag,S,nor)
-!!DEC$ ATTRIBUTES DLLEXPORT,C,REFERENCE,ALIAS:'ssbiv2' :: ssbiv2
+
 !! **************************************************************************
 !! Implementation of the entropy-based dependence measure
 !! proposed by Granger et al. (2004) Journal of Time Series Analysis.
@@ -428,15 +532,16 @@ SUBROUTINE SSBIV2(X,Y,N,nlag,S,nor)
 !! Simone Giannerini DECEMBER 2007
 !!
     USE SHARED_DATA
+    USE ISO_Fortran_env
     IMPLICIT NONE
     INTEGER,intent(IN):: N,nlag,X(N),Y(N),nor
-    REAL(8),intent(OUT)::S(2*nlag+1)
+    REAL(dp),intent(OUT)::S(2*nlag+1)
 
     INTEGER,allocatable:: TX(:,:),TY(:,:),T(:,:)
-    REAL(8)  :: dum
+    REAL(dp)  :: dum
     INTEGER :: k,NX,NY
 
-    S = 999
+    S = 999_dp
     CALL TABF(X,N,TX)
     CALL TABF(Y,N,TY)
     NX = SIZE(TX,1)
@@ -460,7 +565,7 @@ END SUBROUTINE SSBIV2
 !! *************************************************************************************************
 
 SUBROUTINE SSUNIB(X,N,nlag,B,S,M,STAT,nor)
-!!DEC$ ATTRIBUTES DLLEXPORT,C,REFERENCE,ALIAS:'ssunib' :: ssunib
+
 !! ******************************************************************************
 !! PERMUTATION VERSION OF SSUNI
 !! ** INPUT *********************************************************************
@@ -481,25 +586,29 @@ SUBROUTINE SSUNIB(X,N,nlag,B,S,M,STAT,nor)
     IMPLICIT NONE
 INTERFACE
     SUBROUTINE SSUNI(X,N,nlag,S,nor)
+    USE SHARED_DATA
+    USE ISO_Fortran_env
     INTEGER,intent(IN):: N,nlag,X(N),nor
-    REAL(8),intent(OUT)::S(nlag)
+    REAL(dp),intent(OUT)::S(nlag)
     END SUBROUTINE SSUNI
 
     SUBROUTINE SSUNI2(X,N,nlag,S,nor)
+    USE SHARED_DATA
+    USE ISO_Fortran_env
     INTEGER,intent(IN):: N,nlag,X(N),nor
-    REAL(8),intent(OUT)::S(nlag)
+    REAL(dp),intent(OUT)::S(nlag)
     END SUBROUTINE SSUNI2
 END INTERFACE
 
     INTEGER,intent(IN):: N,nlag,X(N),B,STAT,nor
-    REAL(8),intent(OUT):: S(nlag),M(nlag,B)
-    REAL(8):: dum(nlag)
+    REAL(dp),intent(OUT):: S(nlag),M(nlag,B)
+    REAL(dp):: dum(nlag)
     INTEGER:: ind(N),XB(N)
     INTEGER:: i
 
-    S    = 0.;
-    M    = 0.;
-    dum  = 0.;
+    S    = 0.0_dp;
+    M    = 0.0_dp;
+    dum  = 0.0_dp;
     ind  = 0 ;
     XB   = 0 ;
     IF (STAT==0) THEN
@@ -524,7 +633,7 @@ END SUBROUTINE SSUNIB
 !! *************************************************************************************************
 
 SUBROUTINE SSUNI(X,N,nlag,S,nor)
-!!DEC$ ATTRIBUTES DLLEXPORT,C,REFERENCE,ALIAS:'ssuni' :: ssuni
+
 !! **************************************************************************
 !! Implementation of the entropy-based dependence measure
 !! proposed by Granger et al. (2004) Journal of Time Series Analysis.
@@ -534,15 +643,16 @@ SUBROUTINE SSUNI(X,N,nlag,S,nor)
 !! Simone Giannerini March 2007
 !!
     USE SHARED_DATA
+    USE ISO_Fortran_env
     IMPLICIT NONE
     INTEGER,intent(IN):: N,nlag,X(N),nor
-    REAL(8),intent(OUT):: S(nlag)
+    REAL(dp),intent(OUT):: S(nlag)
 
     INTEGER,allocatable:: TX(:,:),TY(:,:),T(:,:)
-    REAL(8)  :: dum
+    REAL(dp)  :: dum
     INTEGER :: k,nx,ny
 
-    S = 0
+    S = 0.0_dp
     DO k = 1,nlag
         CALL TABFD(X(1:(N-k)),X((k+1):N),N-k,TX,TY,T)
         nx   = SIZE(TX,1)
@@ -554,7 +664,7 @@ END SUBROUTINE SSUNI
 !! *************************************************************************************************
 
 SUBROUTINE SSUNI2(X,N,nlag,S,nor)
-!DEC$ ATTRIBUTES DLLEXPORT,C,REFERENCE,ALIAS:'ssuni2' :: ssuni2
+
 !! **************************************************************************
 !! Implementation of the entropy-based dependence measure
 !! proposed by Granger et al. (2004) Journal of Time Series Analysis.
@@ -567,13 +677,14 @@ SUBROUTINE SSUNI2(X,N,nlag,S,nor)
 !! Simone Giannerini 2007
 !!
     USE SHARED_DATA
+    USE ISO_Fortran_env
     IMPLICIT NONE
 
     INTEGER,intent(IN):: N,nlag,X(N),nor
-    REAL(8),intent(OUT):: S(nlag)
+    REAL(dp),intent(OUT):: S(nlag)
 
     INTEGER,allocatable:: TX(:,:),T(:,:)
-    REAL(8) :: dum,smax
+    REAL(dp) :: dum
     INTEGER :: k,NX
 
     CALL TABF(X,N,TX)
@@ -586,14 +697,42 @@ SUBROUTINE SSUNI2(X,N,nlag,S,nor)
     ENDDO
 END SUBROUTINE SSUNI2
 !! *************************************************************************************************
+SUBROUTINE SRhointegrandv(X,nX,x1,x2,N,h1,h2,h1biv,h2biv,SINT)
+! Vectorized version
+    USE SHARED_DATA
+     IMPLICIT NONE
+    INTEGER,INTENT(IN) :: N,nX
+    INTEGER :: i
+    REAL(dp),INTENT(IN) :: X(2,nX),x1(N),x2(N),h1,h2,h1biv,h2biv
+    REAL(dp),INTENT(OUT):: SINT(nX)
+    REAL(dp) :: x1eval(N),x2eval(N),fx1,fx2,fx12,DN1(N),DN2(N)
+!   !# input X is evaluation point for x1 and x2, a 2x1 vector
+do i = 1,nX
+    x1eval(:) = X(1,i)
+    x2eval(:) = X(2,i)
+!# x1 and x2 are the data vectors
+!# Compute the marginal densities
+    CALL DNORMF((x1eval-x1)/h1,N,DN1)
+    fx1 = sum(DN1)/(N*h1)
+    CALL DNORMF((x2eval-x2)/h2,N,DN2)
+    fx2 = sum(DN2)/(N*h2)
+!# Compute the bivariate density
+    CALL DNORMF((x1eval-x1)/h1biv,N,DN1)
+    CALL DNORMF((x2eval-x2)/h2biv,N,DN2)
+    fx12 = (sum(DN1*DN2)/ (N*h1biv*h2biv))
+!# Return the integrand
+    SINT(i) = (sqrt(fx12)-sqrt(fx1)*sqrt(fx2))**2
+enddo
+END SUBROUTINE SRhointegrandv
+!! *************************************************************************************************
 
 SUBROUTINE SRhointegrand(X,x1,x2,N,h1,h2,h1biv,h2biv,SINT)
-!DEC$ ATTRIBUTES DLLEXPORT,C,REFERENCE,ALIAS:'srhointegrand' :: srhointegrand
     USE SHARED_DATA
+     IMPLICIT NONE
     INTEGER,INTENT(IN) :: N
-    REAL(8),INTENT(IN) :: X(2),x1(N),x2(N),h1,h2,h1biv,h2biv
-    REAL(8),INTENT(OUT):: SINT
-    REAL(8) :: x1eval(N),x2eval(N),fx1,fx2,fx12,DN1(N),DN2(N)
+    REAL(dp),INTENT(IN) :: X(2),x1(N),x2(N),h1,h2,h1biv,h2biv
+    REAL(dp),INTENT(OUT):: SINT
+    REAL(dp) :: x1eval(N),x2eval(N),fx1,fx2,fx12,DN1(N),DN2(N)
 !   !# input X is evaluation point for x1 and x2, a 2x1 vector
     x1eval(:) = X(1)
     x2eval(:) = X(2)
@@ -611,17 +750,45 @@ SUBROUTINE SRhointegrand(X,x1,x2,N,h1,h2,h1biv,h2biv,SINT)
     SINT = (sqrt(fx12)-sqrt(fx1)*sqrt(fx2))**2
 END SUBROUTINE SRhointegrand
 !! *************************************************************************************************
+!! *************************************************************************************************
+SUBROUTINE SRhointegrand2(X,nX,xb,N,h1,h2,Hinv,SINT)
+! Vectorized version X is the 2 by nX matrix of points to be evaluated
+! xb is the bivariate time series
+    USE SHARED_DATA
+     IMPLICIT NONE
+    INTEGER,INTENT(IN) :: N,nX
+    INTEGER :: i
+    REAL(dp),INTENT(IN) :: X(2,nX),xb(N,2),h1,h2,Hinv(2,2)
+    REAL(dp),INTENT(OUT):: SINT(nX)
+    REAL(dp) :: xeval(N,2),fx1,fx2,fx12,DN1(N),DN2(N),deth,K(N)
+    deth = Hinv(1,1)*Hinv(2,2) - Hinv(1,2)*Hinv(2,1)
+do i = 1,nX
+    xeval(:,1) = X(1,i)
+    xeval(:,2) = X(2,i)
+! marginal densities
+    CALL DNORMF((xeval(:,1)-xb(:,1))/h1,N,DN1)
+    fx1 = sum(DN1)/(N*h1)
+    CALL DNORMF((xeval(:,2)-xb(:,2))/h2,N,DN2)
+    fx2 = sum(DN2)/(N*h2)
+! bivariate density
+!    SUBROUTINE KGAUSSv(x,d,n,K)
+    CALL KGAUSSv(matmul(Hinv,transpose((xeval-xb))),2,N,K)
+    fx12 = deth*sum(K)/N
+! integrand
+    SINT(i) = (sqrt(fx12)-sqrt(fx1)*sqrt(fx2))**2
+enddo
+END SUBROUTINE SRhointegrand2
+!! *************************************************************************************************
 
 SUBROUTINE kdenestmlcv(X,N,h,F,DMACH)
-!DEC$ ATTRIBUTES DLLEXPORT,C,REFERENCE,ALIAS:'kdenestmlcv' :: kdenestmlcv
 
     USE SHARED_DATA
     IMPLICIT NONE
     INTEGER:: N,i
-    REAL(8) :: X(N),h,F
-    REAL(8) :: fhat(N),LF(N),Xeval(N),d1(1),d2(1),DN(N)
-    REAL(8) :: DMACH(4)
-    d1 = 0
+    REAL(dp) :: X(N),h,F
+    REAL(dp) :: fhat(N),LF(N),Xeval(N),d1(1),d2(1),DN(N)
+    REAL(dp) :: DMACH(4)
+    d1 = 0.0_dp
     CALL DNORMF(d1,1,d2)
     DO i=1,N
         Xeval(:) = X(i)
@@ -643,15 +810,14 @@ END SUBROUTINE kdenestmlcv
 !! *************************************************************************************************
 
 SUBROUTINE kdenestmlcvb(X,Y,N,h,F,DMACH)
-!DEC$ ATTRIBUTES DLLEXPORT,C,REFERENCE,ALIAS:'kdenestmlcvb' :: kdenestmlcvb
     USE SHARED_DATA
     IMPLICIT NONE
 
     INTEGER:: N,i
-    REAL(8) :: X(N),Y(N),h(2),F
-    REAL(8) :: fhat(N),LF(N),Xeval(N),Yeval(N),d1(1),d2(1),DN1(N),DN2(N)
-    REAL(8) :: DMACH(4)
-    d1 = 0
+    REAL(dp) :: X(N),Y(N),h(2),F
+    REAL(dp) :: fhat(N),LF(N),Xeval(N),Yeval(N),d1(1),d2(1),DN1(N),DN2(N)
+    REAL(dp) :: DMACH(4)
+    d1 = 0.0_dp
     CALL DNORMF(d1,1,d2)
     DO i=1,N
         Xeval(:) = X(i)
@@ -674,26 +840,27 @@ SUBROUTINE kdenestmlcvb(X,Y,N,h,F,DMACH)
 END SUBROUTINE kdenestmlcvb
 !! *************************************************************************************************
 
-SUBROUTINE SRhosum(X,x1,x2,N,h1,h2,h1biv,h2biv,S)
-!DEC$ ATTRIBUTES DLLEXPORT,C,REFERENCE,ALIAS:'srhosum' :: srhosum
+SUBROUTINE SRhosum(x1,x2,N,h1,h2,h1biv,h2biv,S)
     USE SHARED_DATA
     IMPLICIT NONE
-
-    INTEGER:: N,i
-    REAL(8) :: X(2),x1(N),x2(N),h1,h2,h1biv,h2biv,S
-!   REAL(8) :: x1eval(N),x2eval(N),fx1,fx2,fx12
-    S = 0
+    INTEGER,INTENT(IN):: N
+    INTEGER :: i
+    REAL(dp),INTENT(IN) :: x1(N),x2(N),h1,h2,h1biv,h2biv
+    REAL(dp) :: X(2)
+    REAL(dp),INTENT(OUT) :: S
+    S = 0.0_dp
     DO i=1,N
         X(1) = x1(i)
         X(2) = x2(i)
-        S = S + (1 - Srho_eval(X,x1,x2,N,h1,h2,h1biv,h2biv))**2
+        S = S + (1 - Srhoeval(X,x1,x2,N,h1,h2,h1biv,h2biv))**2
     ENDDO
-    S = 0.5*S/N
+    S = 0.5_dp*S/N
 CONTAINS
-    FUNCTION Srho_eval(X,x1,x2,N,h1,h2,h1biv,h2biv)
-        INTEGER:: N
-        REAL(8):: X(2),x1(N),x2(N),h1,h2,h1biv,h2biv,Srho_eval
-        REAL(8):: x1eval(N),x2eval(N),fx1,fx2,fx12,DN1(N),DN2(N)
+    FUNCTION Srhoeval(X,x1,x2,N,h1,h2,h1biv,h2biv)
+        INTEGER, INTENT(IN):: N
+        REAL(dp),INTENT(IN) :: X(2),x1(N),x2(N),h1,h2,h1biv,h2biv
+        REAL(dp):: Srhoeval
+        REAL(dp):: x1eval(N),x2eval(N),fx1,fx2,fx12,DN1(N),DN2(N)
         x1eval(:) = X(1)
         x2eval(:) = X(2)
         CALL DNORMF((x1eval-x1)/h1,N,DN1)
@@ -703,35 +870,33 @@ CONTAINS
         CALL DNORMF((x1eval-x1)/h1biv,N,DN1)
         CALL DNORMF((x2eval-x2)/h2biv,N,DN2)
         fx12 = (SUM(DN1*DN2))/ (N*h1biv*h2biv)
-        Srho_eval = SQRT(fx1*fx2/fx12)
-    END FUNCTION Srho_eval
+        Srhoeval = SQRT(fx1*fx2/fx12)
+    END FUNCTION Srhoeval
 END SUBROUTINE SRhosum
 !! *************************************************************************************************
 
 SUBROUTINE SURROGATEACF(A,N,nlag,Te,RT,eps,nsuccmax,nmax,nsurr,che,S)
-!DEC$ ATTRIBUTES DLLEXPORT,C,REFERENCE,ALIAS:'surrogateacf' :: surrogateacf
-
 !   Algoritmo di generazione dei surrogati mediante
 !   annealing simulato (Giugno 1999, rivisto Gennaio 2000, Maggio 2005, AGOSTO 2005)
 !   AGOSTO 2005 VERSIONE PER R
 !   DICEMBRE 2008 VERSIONE PER tseriesEntropy
-!
-!   implicit none
+!   LAST REVISED MAY 2020
+
     USE SHARED_DATA
-
-    REAL(8)  :: RT,Te,T1,eps,DeltaC,p,cost1,cost2,x(3)
+    IMPLICIT NONE
+    REAL(dp)  :: RT,Te,T1,eps,DeltaC,p,cost1,cost2
     INTEGER :: nsucc,nsuccmax,nmax,N,nlag,h,k,che,nsurr,n1,n2,nrep
-    INTEGER :: ind(N)
+    INTEGER :: ind(N),x(2)
 
-    REAL(8) :: Me,Var
-    REAL(8) :: A(N),As(3*N+1),Corig(nlag),Csurr(nlag),Csurr2(nlag),S(N,nsurr)
+    REAL(dp) :: Me,Var
+    REAL(dp) :: A(N),As(3*N+1),Corig(nlag),Csurr(nlag),Csurr2(nlag),S(N,nsurr)
 
 !     Input  Parameters --------------------------------------------------------
 !
 !      N                   ! length of the series
 !      nlag                ! minimization w.r.t to the firts nlag lags
 !      Te                  ! initial temperature
-!      RT                  ! reduction facttor for Te
+!      RT                  ! reduction factor for Te
 !      eps                 ! target tolerance
 !      nsuccmax            ! max number of successes after which Te is lowered
 !      nmax                ! max number of iterations after which Te is lowered
@@ -742,11 +907,11 @@ SUBROUTINE SURROGATEACF(A,N,nlag,Te,RT,eps,nsuccmax,nmax,nsurr,che,S)
     nmax = nmax*N;
     che = che*2*N;
 
-    S = 0
+    S = 0.0_dp
     T1=Te
     nsucc=0
     cost1=10000
-    As = 0
+    As = 0.0_dp
     call Meva(A,N,Me,Var)
     A = A-Me
     As(N+2:2*N+1)=A(1:N)
@@ -762,20 +927,14 @@ SUBROUTINE SURROGATEACF(A,N,nlag,Te,RT,eps,nsuccmax,nmax,nsurr,che,S)
         call ACFsurr(As(N+2:2*N+1),N,nlag,Csurr,Me,Var)   ! funzione di costo del surrogato
 
         cost1=Costo(Corig,Csurr,nlag)
-        CALL RANDOM_seed()
         k=0                                     ! k     : numero delle iterazioni globali
         do while(cost1>=eps)                    !
             h=0                                 ! h     : numero delle iterazioni per ogni serie
             nsucc=0                             ! nsucc : numero di successi per ogni serie
             do while(nsucc<nsuccmax)
-                CALL RANDOM_NUMBER(x)
-                n1=int(N*x(1))+1
-                n2=int(N*x(2))+1
-                do while(n1==n2)            !
-                    CALL RANDOM_NUMBER(x(2))!
-                    n2=int(N*x(2))+1        !       Check per numeri uguali
-                enddo                       !
-
+                x  = BOOTR(N,2)
+                n1 = x(1)
+                n2 = x(2)
                 call ACFswap(Csurr,As,N,n1,n2,nlag,Csurr2,Var)
                 cost2 = Costo(Corig,Csurr2,nlag)
 
@@ -783,7 +942,7 @@ SUBROUTINE SURROGATEACF(A,N,nlag,Te,RT,eps,nsuccmax,nmax,nsurr,che,S)
 
                 if (DeltaC>=0) then
                     p=exp((-DeltaC)/Te)
-                    if(x(3)<=p) then
+                    if(unif_rand()<=p) then
                         nsucc=nsucc+1
                         cost1=cost2
                         Csurr=Csurr2
@@ -810,7 +969,6 @@ SUBROUTINE SURROGATEACF(A,N,nlag,Te,RT,eps,nsuccmax,nmax,nsurr,che,S)
                     As(N+2:2*N+1) = A(ind);
                     call ACFsurr(As(N+2:2*N+1),N,nlag,Csurr,Me,Var)   ! funzione di costo del surrogato
                     cost1=Costo(Corig,Csurr,nlag)
-                    CALL RANDOM_seed()
                 endif
             enddo       !while(nsucc<nsuccmax)
             Te=Te*RT
@@ -828,16 +986,17 @@ CONTAINS
 !! *************************************************************************************************
 
 SUBROUTINE SWAP(x,y)
-    REAL(8),intent(INOUT):: x,y
-    REAL(8) :: tmp
+    IMPLICIT NONE
+    REAL(dp),intent(INOUT):: x,y
+    REAL(dp) :: tmp
     tmp = x; x = y; y = tmp
 end SUBROUTINE SWAP
 !! *************************************************************************************************
 
-REAL(8) FUNCTION COSTO(C1,C2,N)
+REAL(dp) FUNCTION COSTO(C1,C2,N)
     implicit none
     integer,intent(in):: N
-    REAL(8),dimension(N),intent(in):: C1,C2
+    REAL(dp),intent(in):: C1(N),C2(N)
     COSTO = maxval(abs((C1-C2)))
     return
 end FUNCTION Costo
@@ -845,10 +1004,11 @@ end FUNCTION Costo
 
 SUBROUTINE MEVA(X,N,Me,Var)
 !   Calcola media e varianza (corretta) di una serie X
+    IMPLICIT NONE
     INTEGER,intent(in) :: N
-    REAL(8),intent(in) :: X(N)
-    REAL(8),intent(out):: Me,Var
-    REAL(8) :: S2(N),S
+    REAL(dp),intent(in) :: X(N)
+    REAL(dp),intent(out):: Me,Var
+    REAL(dp) :: S2(N),S
 
     S   = SUM(X)
     Me  = S/N              ! Media
@@ -861,16 +1021,16 @@ SUBROUTINE ACFsurr(IN,N,nlag,Rho,Me,Var)
 
 !   Gennaio 2000    Calcola la funzione di Autocorrelazione (Rho)
 !   per i primi nlag tempi su una serie (IN) lunga N, di media Me e varianza Var.
-
-    REAL(8),dimension(N),intent(in):: IN
-    REAL(8),dimension(nlag),intent(out):: Rho
-    REAL(8) :: INd(N) !,S
-    REAL(8) :: Me,Var !,S2,IN2(N)
-
-    ! write(*,*) Me,Var**.5,np,' Media',' sigma',' NN'
-    INd=IN  !-Me        ! togliere il commento per sottrarre la media
+    IMPLICIT NONE
+    INTEGER,intent(in):: N,nlag
+    REAL(dp),intent(in):: IN(N)
+    REAL(dp),intent(out):: Rho(nlag)
+    REAL(dp) :: INd(N) 
+    REAL(dp) :: Me,Var 
+    
+    INd=IN-Me        ! togliere il commento per sottrarre la media
     DO  K=1,nlag
-    Rho(K)=DOT_PRODUCT(INd(K+1:N),INd(1:N-K))
+      Rho(K)=DOT_PRODUCT(INd(K+1:N),INd(1:N-K))
     enddo
     Rho=Rho/(N*Var)
 
@@ -883,15 +1043,16 @@ SUBROUTINE ACFswap(Rho1,As,N,n1,n2,nlag,Rho2,Var)
 !   25/06/01    Parte dalla ACF (Rho1 calcolata su nlag tempi) di una serie in input (As,lunga N)
 !   e calcola la ACF (Rho2) della serie dopo avere scambiato i due elementi di posti n1 e n2.
 !   N.B. As ha N zeri prima e dopo
-!
+!   
+    IMPLICIT NONE
     integer,intent(in) :: n1,n2,nlag,N
-    REAL(8),intent(in) :: Var
-    REAL(8),dimension(nlag),intent(in) :: Rho1
-    REAL(8),dimension(3*N+1),intent(inout) :: As
-    REAL(8),dimension(nlag),intent(out):: Rho2
-    REAL(8),dimension(nlag,4):: b
+    REAL(dp),intent(in) :: Var
+    REAL(dp),intent(in) :: Rho1(nlag)
+    REAL(dp),intent(inout) :: As(3*N+1)
+    REAL(dp),intent(out):: Rho2(nlag)
+    REAL(dp):: b(nlag,4)
 
-    b=0
+    b=0.0_dp
     b(:,1)=As(n1+N+1)*As(n1+N+2:n1+nlag+N+1:1)
     b(:,2)=As(n2+N+1)*As(n2+N+2:n2+nlag+N+1:1)
     b(:,3)=As(n1+N+1)*As(n1+N:n1-nlag+N+1:-1)
@@ -911,5 +1072,5 @@ SUBROUTINE ACFswap(Rho1,As,N,n1,n2,nlag,Rho2,Var)
     Rho2=Rho2/(N*Var)
 end SUBROUTINE ACFswap
 !! *************************************************************************************************
-END SUBROUTINE  SURROGATEACF
+END SUBROUTINE SURROGATEACF
 !! *************************************************************************************************
